"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HlsTech = void 0;
const hls_js_1 = __importDefault(require("hls.js"));
const nas_logger_1 = require("nas-logger");
const utils_1 = require("../utils");
class HlsTech {
    url = '';
    player = null;
    headers = null;
    logger = new nas_logger_1.Logger('HlsTech');
    recover_take = 0;
    is_live = false;
    eventHandler = null;
    autoplay = false;
    onLicenseError = null;
    videoElement = document.createElement('video');
    manifestParsedHandler = (event, data) => {
        data.type = event;
        this.eventHandler(data);
        if (this.autoplay === true) {
            this.videoElement.play();
        }
    };
    levelLoadedHandler = (event, data) => {
        if (data.details != undefined && data.details.type !== 'VOD') {
            this.is_live = true;
        }
        data.type = event;
        this.eventHandler(data);
    };
    errorHandler = (event, data) => {
        data.type = event;
        this.logger.i({ event, data });
        this.eventHandler(data);
        if (data.fatal) {
            switch (data.type) {
                case hls_js_1.default.ErrorTypes.MEDIA_ERROR:
                    this.logger.e("Media error");
                    if (this.recover_take == 1) {
                        this.player.swapAudioCodec();
                    }
                    this.player.recoverMediaError();
                    this.recover_take++;
                    break;
                case hls_js_1.default.ErrorTypes.NETWORK_ERROR:
                    this.logger.e("Network error");
                    this.player.startLoad();
                    break;
                default:
                    this.logger.e("Unrecoverable error");
                    this.destroy();
                    break;
            }
        }
    };
    init(videoElement, url, autoplay, debug, eventHandler, headers, protData, onLicenseError) {
        this.url = url;
        this.headers = headers;
        this.autoplay = autoplay;
        this.logger.debug = debug;
        this.videoElement = videoElement;
        this.onLicenseError = onLicenseError;
        this.videoElement = videoElement;
        this.player = new hls_js_1.default({
            enableWorker: false,
            debug: debug,
            xhrSetup: (xhr, url) => {
                for (var header_name in this.headers) {
                    xhr.setRequestHeader(header_name, this.headers[header_name]);
                }
            }
        });
        this.attachHandlers();
        this.eventHandler = eventHandler;
        this.player.loadSource(this.url);
        this.player.attachMedia(this.videoElement);
    }
    attachHandlers() {
        this.player.on(hls_js_1.default.Events.MANIFEST_PARSED, this.manifestParsedHandler);
        this.player.on(hls_js_1.default.Events.LEVEL_LOADED, this.levelLoadedHandler);
        this.player.on(hls_js_1.default.Events.ERROR, this.errorHandler);
    }
    detachHandlers() {
        this.player.off(hls_js_1.default.Events.MANIFEST_PARSED, this.manifestParsedHandler);
        this.player.off(hls_js_1.default.Events.LEVEL_LOADED, this.levelLoadedHandler);
        this.player.off(hls_js_1.default.Events.ERROR, this.errorHandler);
    }
    getPlayer() {
        return this.player;
    }
    isLive() {
        return this.is_live;
    }
    getAudioTracks() {
        var tracks = this.player.audioTracks;
        var tr = [];
        for (var i = 0; i < tracks.length; i++) {
            tr.push({
                name: tracks[i].name,
                index: i,
                lang: tracks[i].lang
            });
        }
        return tr;
    }
    setAudioTrack(index) {
        this.player.audioTrack = index + 1;
    }
    getQualities() {
        var u = this.player.levels;
        var bitrates = [];
        var b = { index: -1, bitrate: 0, bitrateStr: 'Auto', width: 0, height: 0 };
        bitrates.push(b);
        for (var i = 0; i < u.length; i++) {
            var b = {
                index: 0,
                bitrate: 0,
                bitrateStr: '0k',
                width: 0,
                height: 0,
            };
            b.index = u[i].level != undefined ? u[i].level : i;
            b.bitrate = u[i].bitrate;
            b.bitrateStr = utils_1.Utils.calcBitrateStr(u[i].bitrate);
            b.height = u[i].height;
            bitrates.push(b);
        }
        return bitrates;
    }
    setQuality(index) {
        this.player.currentLevel = index;
    }
    getCurrentQuality() {
        var qualities = this.getQualities();
        for (var i in qualities) {
            if (this.player.currentLevel == qualities[i].index) {
                return qualities[i];
            }
        }
        return {
            index: 0,
            bitrate: 0,
            bitrateStr: '0k',
            height: 0,
            width: 0
        };
    }
    setMaxQuality() {
        var qualities = this.getQualities();
        var maxQualityIndex = -1;
        var bitrate = 0;
        for (var i = 0; i < qualities.length; i++) {
            if (qualities[i].bitrate > bitrate) {
                bitrate = qualities[i].bitrate;
                maxQualityIndex = qualities[i].index;
            }
        }
        this.setQuality(maxQualityIndex);
    }
    destroy() {
        if (this.player != null) {
            this.logger.d("Hlsjs destroy");
            this.detachHandlers();
            this.player.destroy();
            this.player = null;
        }
    }
}
exports.HlsTech = HlsTech;
